const fs = require('fs')
const path = require('path')
const chalk = require('chalk')
const emoji = require('node-emoji')

module.exports = (api, options, rootOptions) => {
  console.log(`\n${emoji.get('pizza')}  Thank you for using ${chalk.cyan('Tailwind / Rollup ESM Component Starter Plugin')}`)
  console.log(`    For questions, issues and recommendations please visit: `)
  console.log(`    ${chalk.green('https://github.com/byte-size/vue-cli-plugin-tailwind-rollup-esm-component')}`)

  const caseHelpers = require('./helpers/case-helpers')
  const postCssHelpers = require('./helpers/postcss-helpers')(api)

  // Remove existing postcss config
  postCssHelpers.getPostCSSConfig()

  // Make sure the input comes in PascalCase and if not, this helper will convert whatever comes to PascalCase
  options.componentNamePascal = caseHelpers.pascalify(options.componentNamePascal)

  // Add kebab-case component name to the options (original options are coming from plugin prompts)
  const kebabCaseName = caseHelpers.kebabcase(options.componentNamePascal)
  Object.assign(options, { componentNameKebab: kebabCaseName })

  // Change default component filename to pascalcase
  fs.rename(
    path.join(__dirname, 'template/src/components/Component.vue'),
    path.join(__dirname, `template/src/components/${options.componentNamePascal}.vue`),
    err => {
      if (err) throw err
    }
  )

  // Install Rollup and supporting packages and add rollup scripts to build
  api.extendPackage({
    main: `dist/${kebabCaseName}.esm.js`,
    devDependencies: {
      tailwindcss: '*',
      rollup: '*',
      'rollup-plugin-buble': '*',
      'rollup-plugin-uglify-es': '*',
      'rollup-plugin-vue': '*',
      '@fullhuman/postcss-purgecss': '*'
    },
    scripts: {
      rollup: `rollup --config src/rollup/rollup.config.js --format es --file dist/${kebabCaseName}.esm.js`
    }
  })

  // Render template
  api.render('./template', options)

  // Modify App.vue to include the new component
  api.onCreateComplete(() => {
    // get content
    const appPath = api.resolve('./src/App.vue')
    let contentMain = fs.readFileSync(appPath, { encoding: 'utf-8' })
    const lines = contentMain.split(/\r?\n/g).reverse()

    // inject Import
    const lastImportIndex = lines.findIndex(line => line.match(/^import/))
    lines[lastImportIndex] += `\n\n// Tailwind\nimport './tailwind/tailwind.css'`
    lines[lastImportIndex] += `\n\nimport ${options.componentNamePascal} from './components/${options.componentNamePascal}.vue'`

    // inject Component
    const lastCompononentIndex = lines.findIndex(line => line.match(/components/))
    lines[lastCompononentIndex] += `\n    ${options.componentNamePascal},`

    // inject HTML Tag
    const lastTagIndex = lines.findIndex(line => line.match(/logo.png/))
    lines[lastTagIndex] += `\n    <${options.componentNamePascal}/>`

    // modify app
    contentMain = lines.reverse().join('\n')
    fs.writeFileSync(appPath, contentMain, { encoding: 'utf-8' })
  })
}
